import FirstTheme from "./themes/FirstTheme";
import {LexicalComposer} from "@lexical/react/LexicalComposer";
import {RichTextPlugin} from "@lexical/react/LexicalRichTextPlugin";
import {ContentEditable} from "@lexical/react/LexicalContentEditable";
import {HistoryPlugin} from "@lexical/react/LexicalHistoryPlugin";
import {AutoFocusPlugin} from "@lexical/react/LexicalAutoFocusPlugin";
import LexicalErrorBoundary from "@lexical/react/LexicalErrorBoundary";
import ToolbarPlugin from "./plugins/ToolbarPlugin";
import {MarkdownShortcutPlugin} from "@lexical/react/LexicalMarkdownShortcutPlugin";
import {
    TRANSFORMERS, $convertFromMarkdownString,
} from "@lexical/markdown";
import "./index.less"
import {importFile} from "../../../utils/File"

import {isEmpty} from "../../../utils/ObjectUtils";
import {ListPlugin} from "@lexical/react/LexicalListPlugin";
import {LinkPlugin} from "@lexical/react/LexicalLinkPlugin";
import AutoLinkPlugin from "./plugins/AutoLinkPlugin";
import ListMaxIndentLevelPlugin from "./plugins/ListMaxIndentLevelPlugin";
import CodeHighlightPlugin from "./plugins/CodeHighlightPlugin";
import ImportFilePlugin from "./plugins/ImportFilePlugin";

import SaveFilePlugin from "./plugins/SaveFilePlugin";
import {TabIndentationPlugin} from "@lexical/react/LexicalTabIndentationPlugin";
import UsefulNodes from "./nodes/UsefulNodes";
import ImagesPlugin from "./plugins/ImagesPlugin";


import {HorizontalRulePlugin} from "@lexical/react/LexicalHorizontalRulePlugin"
import InlineImagePlugin from "./plugins/InlineImagePlugin";
import {TablePlugin} from "@lexical/react/LexicalTablePlugin";
import TableCellActionMenuPlugin from './plugins/TableActionMenuPlugin';
import ExcalidrawPlugin from "./plugins/ExcalidrawPlugin";
import TableOfContentsPlugin from "./plugins/TableOfContentsPlugin";
import ContextMenuPlugin from "./plugins/ContextMenuPlugin"
import {Spin} from "antd";
import {useState} from "react";
import DragDropPaste from "./plugins/DragDropPastePlugin";
function Placeholder() {
    return <div className="editor-placeholder">记录一些灵感吧</div>;
}

export default function Hlexical(props) {
    let editorConfig = {
        theme: FirstTheme,
        onError(error) {
            throw error;
        },
        nodes: UsefulNodes
    };
    const [spinningState,setSpinningState]=useState(!isEmpty(props.filePath))
    console.log("Hlexical(props):this.props.filePath:", props.filePath)
    if (!isEmpty(props.filePath)&&props.filePath.endsWith(".md")){
        let promise = importFile(props.filePath);
        promise.then(value => {
            if (isEmpty(value)) {
                setSpinningState(false)
                return
            }
            editorConfig={...editorConfig,editorState: () => $convertFromMarkdownString(value.toString(), TRANSFORMERS)}
            setSpinningState(false)
        }).catch(error =>
            console.error(error)
        )
    }
    return (

        <LexicalComposer initialConfig={editorConfig}>
            <div className="editor-container">
                {/* 富文本插件 */}
                <ToolbarPlugin/>
                <div className="editor-inner">
                    <Spin size="large" fullscreen="ture" spinning={spinningState}>
                    <RichTextPlugin
                        contentEditable={<ContentEditable className="editor-input"/>}
                        placeholder={<Placeholder/>}
                        ErrorBoundary={LexicalErrorBoundary}
                    />    </Spin>
                    <HistoryPlugin/>
                    {/*黑窗口动态记录当前操作*/}
                    {/*<TreeViewPlugin/>*/}
                    <AutoFocusPlugin/>
                    <CodeHighlightPlugin/>
                    <ListPlugin/>
                    <LinkPlugin/>
                    <AutoLinkPlugin/>
                    <ListMaxIndentLevelPlugin maxDepth={7}/>
                    {/* 表格操作 */}
                    <TablePlugin
                        hasCellMerge={true}
                        hasCellBackgroundColor={true}
                    />
                    {/* 表格单元格操作 */}
                    <TableCellActionMenuPlugin/>
                    <TabIndentationPlugin/>
                    {/*markdown 快捷键*/}
                    <MarkdownShortcutPlugin transformers={TRANSFORMERS}/>
                    {/*图片加载*/}
                    <ImagesPlugin/>
                    <InlineImagePlugin/>

                    {/*分割线 */}
                    <HorizontalRulePlugin/>
                    {/*页分割线*/}

                    {/*目录加载*/}
                    <TableOfContentsPlugin filePath={props.filePath}/>
                    {/*右键菜单*/}
                    <ContextMenuPlugin/>
                    {/* 画图 */}
                    <ExcalidrawPlugin/>
                    {/* 拖拽复制*/}
                    <DragDropPaste />
                    <ImportFilePlugin filePath={props.filePath} setSpinningState={setSpinningState}/>
                    <SaveFilePlugin filePath={props.filePath}/>
                    {/*文件操作导入文件*/}
                    {/*<ActionPlugin/>*/}
                </div>
            </div>

        </LexicalComposer>

    );
}
